//  `````                     +yys.                              ````````````                                         
// yNNNNNNmds:                dNNm:                 `++/        `mNNNNNNNNNNN`                                        
// yNNNdyhmNNNo    .://:.     -//:` `---  -:/:.    `hNNd---      syyyyydNNNNy    .://:-    `---  .:/.   .://:.        
// yNNNs  sNNNd  +mNNNNNNmo`  mNNN: /NNNymNNNNNh. ymNNNNNNd           +mNNm/   +dNNNNNNms` :NNNohNNN: +dNNNNNNms`     
// yNNNhosmNNNo sNNNh--sNNNh  mNNN: /NNNNs:/mNNNs :sNNNd::-         .hNNNh.   oNNNh.`:mNNh :NNNNmyss`sNNNh:-sNNNd`    
// yNNNNNNNNh/  NNNN:  .NNNN- mNNN: /NNNm   sNNNy  +NNNh           /mNNm+     mNNNmmmmNNNm :NNNN.    mNNN/  `NNNN:    
// yNNNy--.`    mNNN+  -NNNN. mNNN: /NNNd   sNNNy  +NNNh         `sNNNd-      dNNNy//////: :NNNN     dNNN+  .NNNN-    
// yNNNs        :mNNmysmNNN+  mNNN: /NNNd   sNNNy  /NNNNhyd     `dNNNNmdddddd--mNNNyoosyd- :NNNN     -mNNmysmNNNo     
// sddd+         `+ydmmdho.   hddd- :dddy   oddds   +hmmmdy     .dddddddddddd. `/ydmmmdhs` -dddd      `+ydmmdho-   
//   
// ---------------------------------------------------------------------------------------------------------------
// File        | PZ_SupportResistance.mq4
// Description | Evaluates support and resistance using fractals and the Zig Zag indicator.
// Copyright   | Point Zero Trading Solutions
// Website     | http://www.pointzero-trading.com        
// --------------------------------------------------------------------------------------------------------------- 
#property copyright "Point Zero Trading Solutions"
#property link      "http://www.pointzero-trading.com"

//---- Buffer properties
#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 DodgerBlue
#property indicator_color2 Gold
#property indicator_width1 1
#property indicator_width2 1

//---- Constants
#define ZZBack             1                    
#define ZZDev              5                    

//-------------------------------
// Input parameters
//-------------------------------
extern int  ZigZagFast     = 6;
extern int  ZigZagSlow     = 24;

//-------------------------------
// Buffers
//-------------------------------
double v1[];
double v2[];
double middle[];

//-------------------------------
// Internal variables
//-------------------------------

// ZigZag stores values
double zz_slow_high = 0;
double zz_slow_low = 0;
double zz_fast_high = 0;
double zz_fast_low = 0;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   // Buffers
   IndicatorBuffers(2);
   SetIndexArrow(0, 159); SetIndexStyle(0, DRAW_ARROW, STYLE_DOT);
   SetIndexBuffer(0, v1);
   SetIndexLabel(0,"Resistance");
   SetIndexArrow(1, 159); SetIndexStyle(1, DRAW_ARROW, STYLE_DOT);
   SetIndexBuffer(1, v2);
   SetIndexLabel(1,"Support");
   
   // My name
   IndicatorShortName("PZ Support Resistance");
   Comment("Copyright  http://www.pointzero-trading.com");
   
   // Bye!
   return(0); 
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   // Start, limit, etc..
   int start = 1;
   int limit;
   int counted_bars = IndicatorCounted();

   // nothing else to do?
   if(counted_bars < 0) 
       return(-1);

   // do not check repeated bars
   limit = Bars - 1 - counted_bars;
    
   // Iterate from past to present
   for(int i = limit; i >= start; i--)
   {
      // Last fractals
      double val1 = upper_fractal(i);
      double val2 = lower_fractal(i);
      
      // Slow zigzag values
      double zz_slow_high_t = iCustom(Symbol(), 0, "ZigZag", ZigZagSlow, ZZDev, ZZBack, 1, i);
      if(zz_slow_high_t != 0) zz_slow_high = zz_slow_high_t;
        
      // Zig Zag low
      double zz_slow_low_t  = iCustom(Symbol(), 0, "ZigZag", ZigZagSlow, ZZDev, ZZBack, 2, i);
      if(zz_slow_low_t != 0) zz_slow_low = zz_slow_low_t;
      
      // Slow zigzag values
      double zz_fast_high_t = iCustom(Symbol(), 0, "ZigZag", ZigZagFast, ZZDev, ZZBack, 1, i);
      if(zz_fast_high_t != 0) zz_fast_high = zz_fast_high_t;
        
      // Zig Zag low
      double zz_fast_low_t  = iCustom(Symbol(), 0, "ZigZag", ZigZagFast, ZZDev, ZZBack, 2, i);
      if(zz_fast_low_t != 0) zz_fast_low = zz_fast_low_t;
      
      // Upper Fractal
      if (val1 > 0 && (val1 == zz_slow_high || val1 == zz_fast_high)) 
      {
         v1[i]   = val1;
         v1[i+1] = val1;
         v1[i+2] = val1;
      } else {
         v1[i] = v1[i+1];
      }
    
      // Lower Fractal
      if (val2 > 0 && (val2 == zz_slow_low || val2 == zz_fast_low)) 
      {
         v2[i]   = val2;
         v2[i+1] = val2;
         v2[i+2] = val2;
      } else {
         v2[i] = v2[i+1];
      }  
      
      // Save middle
      middle[i] = (v1[i] + v2[i]) / 2;
   }
   return(0);
}
  
/**
* Returns fractal resistance
* @param int shift
* @param int bk
*/
double upper_fractal(int shift = 1)
{
   double middle = iHigh(Symbol(), 0, shift + 2);
   double v1 = iHigh(Symbol(), 0, shift);
   double v2 = iHigh(Symbol(), 0, shift+1);
   double v3 = iHigh(Symbol(), 0, shift + 3);
   double v4 = iHigh(Symbol(), 0, shift + 4);
   if(middle > v1 && 
      middle > v2 && 
      middle > v3 && 
      middle > v4
      //&& v2 > v1 && v3 > v4 // Uncomment me for perfect fractals
      )
   {
      return(middle);
   }
   return(0);
}

/**
* Returns fractal support and stores wether it has changed or not
* @param int shift
*/

double lower_fractal(int shift = 1)
{
   double middle = iLow(Symbol(), 0, shift + 2);
   double v1 = iLow(Symbol(), 0, shift);
   double v2 = iLow(Symbol(), 0, shift+1);
   double v3 = iLow(Symbol(), 0, shift + 3);
   double v4 = iLow(Symbol(), 0, shift + 4);
   if(middle < v1 && 
      middle < v2 && 
      middle < v3 && 
      middle < v4
      //&& v2 < v1 && v3 < v4  //Uncomment me for perfect fractals
      )
   {
      return(middle);
   }
   return(0);
}